//handles 256-color PIXELARRAY structure
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "bmp.h"
#include "util0.h"

int PixelArrayCreate(PIXELARRAY*ppa,int xs,int ys)
{void*pv;

  FillMemory(ppa,sizeof(PIXELARRAY),0);
  pv=GetMem(xs*ys);
  if (NULL==pv) return -1;
  ppa->paX=xs; ppa->paY=ys; ppa->ppaArray=pv;
  return 0;
}

void PixelArrayDelete(PIXELARRAY*ppa)
{
  if (ppa->ppaArray) FreeMem(ppa->ppaArray);
  FillMemory(ppa,sizeof(PIXELARRAY),0);
}

void PixelArray256ClrToGray(PIXELARRAY*ppa,RGBQUAD*pPal)
//convert 256-color pixel array to grayscale
{BYTE*pb,grays[256];
 int ii;

  for (pb=grays,ii=0;ii<256;ii++) {
    //use average or RGB now, should convert to the real
    //color-to-grayscale formular later
    pb[0]=(pPal->rgbBlue + pPal->rgbGreen + pPal->rgbRed)/3;
    pb++; pPal++;
  }
  pb=ppa->ppaArray;
  for (ii=ppa->paX*ppa->paY;ii;ii--) {
    pb[0]=grays[pb[0]]; pb++;
  }
}

int PixelArrayLoad256ClrBmp(PIXELARRAY*ppa,RGBQUAD*pPal,char*pfn)
/*load a 256-color BMP into PIXELARRAY
1. open file
2. verify format
3. get line width and image size in bytes
4. if line width not a multiple of 4 then exit
5. allocate memory
6. read palette if caller supply a palette pointer
7. read pixels
8. convert bottom-up to top down if necessary
*/
{BITMAPFILEHEADER bfh; BITMAPINFOHEADER bih;
 HANDLE hf;
 DWORD byteread;
 int retv;
 long xs,ys,lineadv,l2;
 BYTE*pstrt;

  hf=CreateFile(pfn,GENERIC_READ,FILE_SHARE_READ,NULL,
                OPEN_EXISTING,FILE_FLAG_SEQUENTIAL_SCAN,NULL);
  if (INVALID_HANDLE_VALUE==hf) return -1;

  ReadFile(hf,&bfh,sizeof(BITMAPFILEHEADER),&byteread,NULL);
  ReadFile(hf,&bih,sizeof(BITMAPINFOHEADER),&byteread,NULL);
 
  retv=-1; //assume failure
  
  if (0x4d42!=bfh.bfType) goto l256cb90; //wrong signature
  //check planes, bit count, and compression
  if (1!=bih.biPlanes || 8!=bih.biBitCount ||
      BI_RGB!=bih.biCompression) goto l256cb90;
  //check color usage
  if (bih.biClrUsed>256) goto l256cb90;
  if (0==bih.biClrUsed) bih.biClrUsed=256;
  //check size, and line width must be a multiple of 4
  xs=bih.biWidth; ys=bih.biHeight; 
  if (ys<0) ys=-ys;
  //if too big or too small
  if (xs<4 ||ys<4 || xs>65535 || ys>65535) goto l256cb90;  
  if (xs & 3) goto l256cb90; //width not multiple of 4

  if (pPal) { //if caller wants palette
    ReadFile(hf,pPal,bih.biClrUsed*sizeof(RGBQUAD),&byteread,NULL);
    retv=-2; //indicate palette is read
  }

  if (PixelArrayCreate(ppa,xs,ys)) goto l256cb90;
  SetFilePointer(hf,bfh.bfOffBits,NULL,FILE_BEGIN);

  pstrt=ppa->ppaArray; lineadv=xs;
  if (bih.biHeight>0) { //if bottom-up bitmap
    lineadv=-lineadv; //we always load in top-down format
    pstrt+=xs*(ys-1);
  }
  for (l2=0;l2<ys;l2++) {
    ReadFile(hf,pstrt,xs,&byteread,NULL);
    pstrt+=lineadv;
  }
  
  retv=0;
l256cb90:
  CloseHandle(hf); return retv;
}

